=== modified file 'lib/lp/services/messages/model/message.py'
--- lib/lp/services/messages/model/message.py	2011-08-12 14:39:51 +0000
+++ lib/lp/services/messages/model/message.py	2011-08-15 19:14:22 +0000
@@ -27,6 +27,7 @@
     parseaddr,
     parsedate_tz,
     )
+import logging
 from operator import attrgetter
 import os.path
 
@@ -207,6 +208,10 @@
 class MessageSet:
     implements(IMessageSet)
 
+    extra_encoding_aliases = {
+        'macintosh': 'mac_roman',
+    }
+
     def get(self, rfc822msgid):
         messages = list(Message.selectBy(rfc822msgid=rfc822msgid))
         if len(messages) == 0:
@@ -231,6 +236,19 @@
         Store.of(message).flush()
         return message
 
+    @classmethod
+    def decode(self, encoded, encoding):
+        encoding = self.extra_encoding_aliases.get(encoding, encoding)
+        try:
+            return encoded.decode(encoding, 'replace')
+        except LookupError:
+            try:
+                return encoded.decode('us-ascii')
+            except UnicodeDecodeError:
+                logging.getLogger().warning(
+                    'Treating unknown encoding "%s" as latin-1.' % encoding)
+                return encoded.decode('latin-1')
+
     def _decode_header(self, header):
         r"""Decode an RFC 2047 encoded header.
 
@@ -264,7 +282,7 @@
             # cause problems in unusual encodings that we are hopefully
             # unlikely to encounter in this part of the code.
             re_encoded_bits.append(
-                (bytes.decode(charset, 'replace').encode('utf-8'), 'utf-8'))
+                (self.decode(bytes, charset).encode('utf-8'), 'utf-8'))
 
         return unicode(email.Header.make_header(re_encoded_bits))
 
@@ -441,8 +459,7 @@
                 charset = part.get_content_charset()
                 if charset is None or str(charset).lower() == 'x-unknown':
                     charset = 'latin-1'
-
-                content = content.decode(charset, 'replace')
+                content = self.decode(content, charset)
 
                 if content.strip():
                     MessageChunk(

=== modified file 'lib/lp/services/messages/tests/test_message.py'
--- lib/lp/services/messages/tests/test_message.py	2011-05-31 20:59:10 +0000
+++ lib/lp/services/messages/tests/test_message.py	2011-08-16 13:35:41 +0000
@@ -4,14 +4,14 @@
 __metaclass__ = type
 
 from cStringIO import StringIO
-from email.Message import Message
-from email.MIMEMultipart import MIMEMultipart
-from email.MIMEText import MIMEText
-from email.Utils import (
+from email.header import Header
+from email.message import Message
+from email.mime.multipart import MIMEMultipart
+from email.mime.text import MIMEText
+from email.utils import (
     formatdate,
     make_msgid,
     )
-import unittest
 
 from sqlobject import SQLObjectNotFound
 import transaction
@@ -29,17 +29,22 @@
     )
 from lp.services.job.model.job import Job
 from lp.services.mail.sendmail import MailController
-from lp.testing import TestCaseWithFactory
+from lp.testing import (
+    TestCase,
+    TestCaseWithFactory,
+    )
 from lp.testing.factory import LaunchpadObjectFactory
 
 
-class TestMessageSet(unittest.TestCase):
+class TestMessageSet(TestCase):
     """Test the methods of `MessageSet`."""
 
     layer = LaunchpadFunctionalLayer
 
+    high_characters = ''.join(chr(c) for c in range(128, 256))
+
     def setUp(self):
-        unittest.TestCase.setUp(self)
+        super(TestMessageSet, self).setUp()
         # Testing behavior, not permissions here.
         login('foo.bar@canonical.com')
         self.factory = LaunchpadObjectFactory()
@@ -149,6 +154,53 @@
         dupe_message = MessageSet().fromEmail(email.as_string())
         self.assertNotEqual(orig_message.id, dupe_message.id)
 
+    def makeEncodedEmail(self, encoding_name, actual_encoding):
+        email = self.factory.makeEmailMessage(body=self.high_characters)
+        email.set_type('text/plain')
+        email.set_charset(encoding_name)
+        macroman = Header(self.high_characters, actual_encoding).encode()
+        new_subject = macroman.replace(actual_encoding, encoding_name)
+        email.replace_header('Subject', new_subject)
+        return email
+
+    def test_fromEmail_decodes_macintosh_encoding(self):
+        """"macintosh encoding is equivalent to MacRoman."""
+        high_decoded = self.high_characters.decode('macroman')
+        email = self.makeEncodedEmail('macintosh', 'macroman')
+        message = MessageSet().fromEmail(email.as_string())
+        self.assertEqual(high_decoded, message.subject)
+        self.assertEqual(high_decoded, message.text_contents)
+
+    def test_fromEmail_decodes_booga_encoding(self):
+        """"'booga' encoding is decoded as latin-1."""
+        high_decoded = self.high_characters.decode('latin-1')
+        email = self.makeEncodedEmail('booga', 'latin-1')
+        message = MessageSet().fromEmail(email.as_string())
+        self.assertEqual(high_decoded, message.subject)
+        self.assertEqual(high_decoded, message.text_contents)
+
+    def test_decode_utf8(self):
+        """Test decode with a known encoding."""
+        result = MessageSet.decode(u'\u1234'.encode('utf-8'), 'utf-8')
+        self.assertEqual(u'\u1234', result)
+
+    def test_decode_macintosh(self):
+        """Test decode with macintosh encoding."""
+        result = MessageSet.decode(self.high_characters, 'macintosh')
+        self.assertEqual(self.high_characters.decode('macroman'), result)
+
+    def test_decode_unknown_ascii(self):
+        """Test decode with ascii characters in an unknown encoding."""
+        result = MessageSet.decode('abcde', 'booga')
+        self.assertEqual(u'abcde', result)
+
+    def test_decode_unknown_high_characters(self):
+        """Test decode with non-ascii characters in an unknown encoding."""
+        with self.expectedLog(
+            'Treating unknown encoding "booga" as latin-1.'):
+            result = MessageSet.decode(self.high_characters, 'booga')
+        self.assertEqual(self.high_characters.decode('latin-1'), result)
+
 
 class TestMessageJob(TestCaseWithFactory):
     """Tests for MessageJob."""

